#include <unistd.h>

// Директива говорит компилятору, что блок код должен быть скомпилирован для CPU и сопроцессора
// Начало блока двойной компиляции
#pragma offload_attribute(push, target(mic))

#include <stdio.h>
#include <omp.h>

void testThreadCount()
{
    int thread_count;

    #pragma omp parallel
    {
        #pragma omp single
            thread_count = omp_get_num_threads();
    }
    printf("Number of threads: %d\n", thread_count);
}

// Конец блока кода двойной компиляции
#pragma offload_attribute(pop)

int main()
{
    // Вывод имени узла
    char hostname[1024];
    gethostname(hostname, 1024);
    printf("Host name: %s\n", hostname);
            
    // Вывод количества потоков,выполняющихся на центральном процессоре
    printf("CPU:\n");
    testThreadCount();

	// Получение количества доступных сопроцессоров
    int number_of_coprocessors = _Offload_number_of_devices();

    printf("Intel Xeon Phi:\n");
    printf("Number of coprocessors: %d\n", number_of_coprocessors);
    fflush(stdout);

    for (int i = 0; i < number_of_coprocessors; ++i)
    {
	// Вывод количества потоков на сопроцессоре с номером i
        #pragma offload target(mic:i)
        {
            testThreadCount();
        }
    }

    return 0;
}
