#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

// Директива говорит компилятору, что блок код должен быть скомпилирован для CPU и сопроцессора
// Начало блока двойной компиляции
#pragma offload_attribute(push, target(mic))

// Параллельное вычисление скалярного произведения векторов
float dot(float* a, float* b, int n)
{
    float res = 0;

    #pragma omp parallel for reduction(+: res)
    for (int i = 0; i < n; ++i)
    {
        res += a[i]*b[i];
    }

    return res;
}

// Конец блока кода двойной компиляции
#pragma offload_attribute(pop)

int main()
{
    // Вывод имени узла
    char hostname[1024];
    gethostname(hostname, 1024);
    printf("Host name: %s\n", hostname);
                
    int n = 100;
    float* a = new float[n];
    float* b = new float[n];
    float res_cpu, res_mic;

    for (int i = 0; i < n; ++i)
    {
        a[i] = (float)rand()/RAND_MAX;
        b[i] = (float)rand()/RAND_MAX;
    }

    // Вычисление скалярного произведения на процессоре
    res_cpu = dot(a, b, n);
    
    printf("CPU dot: %f\n", res_cpu);

    // Вычисление скалярного произведения на сопроцессоре.
    // Параметр in гооврит о необходимости копирования
    // массивов a и b на сопроцессор. Обратно массивы не копируются.
    #pragma offload target(mic) in(a[0:n], b[0:n])
    res_mic = dot(a, b, n);
    
    printf("MIC dot: %f\n", res_mic);

    delete[] a;
    delete[] b;

    return 0;
}
